﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Security.Cryptography.X509Certificates;
using System.IO;
using System.Net;
using System.Web;
using System.Xml;

namespace Rapida
{
    public class Rapida
    {
        public string Host;
        public string Certificate;
        public bool X509FromFileOrStore;
        public string Function;
        public string PaymExtId;
        public int Code;
        public int PaymSubjTp;
        public string TermType;
        public string TermId;
        public string Params;
        public int Amount;
        public int FeeSum;
        public string Request;
        public string Response;
        public int LastErrorCode;
        public string LastErrorDescription;

        private X509Certificate certificate;

        private void GetError(string error_code)
        {
            switch (error_code)
            {
                case "0":
                    this.LastErrorCode = 0;
                    this.LastErrorDescription = "Нет ошибок";
                    break;
                case "-1":
                    this.LastErrorCode = -1;
                    this.LastErrorDescription = "Входящее свойство не задано";
                    break;
                case "-2":
                    this.LastErrorCode = -2;
                    this.LastErrorDescription = "Ошибка исходящего запроса";
                    break;
                case "-3":
                    this.LastErrorCode = -3;
                    this.LastErrorDescription = "Ошибка при получении ответа";
                    break;
                case "-4":
                    this.LastErrorCode = -4;
                    this.LastErrorDescription = "Ошибка при подключении сертификата";
                    break;
                case "-5":
                    this.LastErrorCode = -5;
                    this.LastErrorDescription = "Сертификат не найден";
                    break;
                case "-6":
                    this.LastErrorCode = -6;
                    this.LastErrorDescription = "Ошибка при получении идентификатора запроса";
                    break;
                case "-9":
                    this.LastErrorCode = -9;
                    this.LastErrorDescription = "Неизвестная ошибка";
                    break;
            }
        }

        private void SendRequest()
        {
            try
            {
                HttpWebRequest HTTPRequest;

                try
                {                    
                    HTTPRequest = (HttpWebRequest)WebRequest.Create(this.Host + this.Request);
                    HTTPRequest.ClientCertificates.Add(this.certificate);                    
                }
                catch
                {
                    throw new Exception("-2");
                }

                try
                {
                    HttpWebResponse response = (HttpWebResponse)HTTPRequest.GetResponse();
                    Stream stream = response.GetResponseStream();
                    TextReader xml = new StreamReader(stream, Encoding.GetEncoding(1251));

                    this.Response = xml.ReadToEnd();
                    stream.Close();
                    response.Close();
                }
                catch
                {
                    throw new Exception("-3");
                }

                GetError("0");
            }
            catch (Exception ex)
            {
                GetError(ex.Message);
                this.Response = "ОШИБКА!";
            }
        }

        public void GetCertificate()
        {            
            GetError("-5");

            if (this.X509FromFileOrStore)
            {
                // Not available
            }
            else
            {
                System.Net.ServicePointManager.CertificatePolicy = new TrustAllCertificatePolicy();
                X509Store store = new X509Store(StoreLocation.CurrentUser);
                try
                {
                    store.Open(OpenFlags.ReadOnly);
                    this.certificate = new X509Certificate();

                    string domain;
                    foreach (X509Certificate crt in store.Certificates)
                    {
                        domain = crt.Subject;
                        domain = domain.Substring(domain.IndexOf("E=") + 2, domain.Length - domain.IndexOf("E=") - 2);

                        if (domain.IndexOf(",") > 0)
                        {
                            domain = domain.Substring(0, domain.IndexOf(","));
                        }

                        if (domain == this.Certificate)
                        {
                            this.certificate = crt;
                            GetError("0");
                            break;
                        }
                    }                    
                }
                catch
                {
                    GetError("-4");
                }
                finally
                {
                    store.Close();
                }
            }
        }

        public void PaymExtIdGen()
        {
            try
            {
                Random rnd = new Random();
                string uin = "";
                for (int i = 0; i < 20; i++)
                {
                    uin += rnd.Next(10).ToString();
                }

                this.PaymExtId = uin;

                GetError("0");
            }
            catch
            {
                GetError("-6");
            }
        }

        public void Check()
        {
            try
            {
                this.Function = "check";
                this.Params = HttpUtility.UrlEncode((this.Params), Encoding.GetEncoding(1251));

                if ((this.Function.Length > 0) &&
                    (this.PaymExtId.Length > 0) &&
                    (this.PaymSubjTp.ToString().Length > 0) &&
                    (this.Amount.ToString().Length > 0) &&
                    (this.Params.Length > 0) &&
                    (this.TermType.Length > 0) &&
                    (this.TermId.Length > 0) &&
                    (this.FeeSum.ToString().Length > 0))
                {
                    this.Request = string.Format("function={0}&PaymExtId={1}&PaymSubjTp={2}&Amount={3}&Params={4}&TermType={5}&TermID={6}&FeeSum={7}",
                                                 this.Function, this.PaymExtId, this.PaymSubjTp.ToString(), this.Amount.ToString(), this.Params, this.TermType, this.TermId, this.FeeSum.ToString());
                    SendRequest();
                }
                else
                {
                    GetError("-1");
                }                
            }
            catch
            {
                GetError("-9");
            }
        }

        public void Payment()
        {
            try
            {
                this.Function = "payment";
                this.Params = HttpUtility.UrlEncode((this.Params), Encoding.GetEncoding(1251));

                if ((this.Function.Length > 0) &&
                        (this.PaymExtId.Length > 0) &&
                        (this.PaymSubjTp.ToString().Length > 0) &&
                        (this.Amount.ToString().Length > 0) &&
                        (this.Params.Length > 0) &&
                        (this.TermType.Length > 0) &&
                        (this.TermId.Length > 0) &&
                        (this.FeeSum.ToString().Length > 0))
                {
                    this.Request = string.Format("function={0}&PaymExtId={1}&PaymSubjTp={2}&Amount={3}&Params={4}&TermType={5}&TermID={6}&FeeSum={7}",
                                                 this.Function, this.PaymExtId, this.PaymSubjTp.ToString(), this.Amount.ToString(), this.Params, this.TermType, this.TermId, this.FeeSum.ToString());
                    SendRequest();
                }
                else
                {
                    GetError("-1");
                }
            }
            catch
            {
                GetError("-9");
            }
        }

        public void Cancel()
        {
            try
            {
                if ((this.Function.Length > 0) &&
                        (this.PaymExtId.Length > 0))
                {
                    this.Function = "cancelreq";
                    this.Request = string.Format("function={0}&PaymExtId={1}",
                                                 this.Function, this.PaymExtId);
                    SendRequest();
                }
                else
                {
                    GetError("-1");
                }
            }
            catch
            {
                GetError("-9");
            }
        }

        public void GetPaymSubj()
        {
            try
            {
                this.Function = "getpaymsubj";
                this.Params = HttpUtility.UrlEncode((this.Params), Encoding.GetEncoding(1251));

                if ((this.Function.Length > 0) &&
                        (this.Code.ToString().Length > 0) &&
                        (this.Params.Length > 0))
                {
                    this.Request = string.Format("function={0}&Code={1}&Params={2}",
                                                  this.Function, this.Code, this.Params);
                    SendRequest();
                }
                else
                {
                    GetError("-1");
                }
            }
            catch
            {
                GetError("-9");
            }
        }

        public void GetBalance()
        {
            try
            {
                this.Function = "getbalance";

                if ((this.Function.Length > 0) &&
                        (this.PaymExtId.Length > 0))
                {
                    this.Request = string.Format("function={0}&PaymExtId={1}",
                                                 this.Function, this.PaymExtId);
                    SendRequest();
                }
                else
                {
                    GetError("-1");
                }
            }
            catch
            {
                GetError("-9");
            }
        }

        public void GetFee()
        {
            try
            {
                this.Function = "getfee";

                if (this.Function.Length > 0)
                {
                    this.Request = string.Format("function={0}",
                                                 this.Function);
                    SendRequest();
                }
                else
                {
                    GetError("-1");
                }
            }
            catch
            {
                GetError("-9");
            }
        }

        public void GetStatus()
        {
            try
            {
                this.Function = "getstatus";
                if ((this.Function.Length > 0) &&
                        (this.PaymExtId.Length > 0))
                {
                    this.Request = string.Format("function={0}&PaymExtId={1}",
                                                 this.Function, this.PaymExtId);
                    SendRequest();
                }
                else
                {
                    GetError("-1");
                }
            }
            catch
            {
                GetError("-9");
            }
        }
    }

    // Класс автоматического подтверждения используемого сертификата
    #region class TrustAllCertificatePolicy : System.Net.ICertificatePolicy
    class TrustAllCertificatePolicy : System.Net.ICertificatePolicy
    {
        public TrustAllCertificatePolicy()
        {
        }

        public bool CheckValidationResult(System.Net.ServicePoint sp, X509Certificate cert, System.Net.WebRequest req, int problem)
        {
            return true;
        }
    }
    #endregion
}
